"""Auto generated, do not edit."""
# pylint: disable=line-too-long,too-many-lines,too-many-arguments,missing-class-docstring,missing-function-docstring
import datetime
import collections.abc
import _zivid
import zivid
import zivid._settings_converter


class Settings:
    class Acquisition:
        def __init__(
            self,
            aperture=_zivid.Settings().Acquisition().Aperture().value,
            brightness=_zivid.Settings().Acquisition().Brightness().value,
            exposure_time=_zivid.Settings().Acquisition().ExposureTime().value,
            gain=_zivid.Settings().Acquisition().Gain().value,
        ):

            if isinstance(aperture, (float, int,)) or aperture is None:
                self._aperture = _zivid.Settings.Acquisition.Aperture(aperture)
            else:
                raise TypeError(
                    "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                        value_type=type(aperture)
                    )
                )
            if isinstance(brightness, (float, int,)) or brightness is None:
                self._brightness = _zivid.Settings.Acquisition.Brightness(brightness)
            else:
                raise TypeError(
                    "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                        value_type=type(brightness)
                    )
                )
            if (
                isinstance(exposure_time, (datetime.timedelta,))
                or exposure_time is None
            ):
                self._exposure_time = _zivid.Settings.Acquisition.ExposureTime(
                    exposure_time
                )
            else:
                raise TypeError(
                    "Unsupported type, expected: (datetime.timedelta,) or None, got {value_type}".format(
                        value_type=type(exposure_time)
                    )
                )
            if isinstance(gain, (float, int,)) or gain is None:
                self._gain = _zivid.Settings.Acquisition.Gain(gain)
            else:
                raise TypeError(
                    "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                        value_type=type(gain)
                    )
                )

        @property
        def aperture(self):
            return self._aperture.value

        @property
        def brightness(self):
            return self._brightness.value

        @property
        def exposure_time(self):
            return self._exposure_time.value

        @property
        def gain(self):
            return self._gain.value

        @aperture.setter
        def aperture(self, value):
            if isinstance(value, (float, int,)) or value is None:
                self._aperture = _zivid.Settings.Acquisition.Aperture(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: float or  int or None, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        @brightness.setter
        def brightness(self, value):
            if isinstance(value, (float, int,)) or value is None:
                self._brightness = _zivid.Settings.Acquisition.Brightness(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: float or  int or None, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        @exposure_time.setter
        def exposure_time(self, value):
            if isinstance(value, (datetime.timedelta,)) or value is None:
                self._exposure_time = _zivid.Settings.Acquisition.ExposureTime(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: datetime.timedelta or None, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        @gain.setter
        def gain(self, value):
            if isinstance(value, (float, int,)) or value is None:
                self._gain = _zivid.Settings.Acquisition.Gain(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: float or  int or None, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        def __eq__(self, other):
            if (
                self._aperture == other._aperture
                and self._brightness == other._brightness
                and self._exposure_time == other._exposure_time
                and self._gain == other._gain
            ):
                return True
            return False

        def __str__(self):
            return str(zivid._settings_converter.to_internal_settings_acquisition(self))

    class Processing:
        class Color:
            class Balance:
                def __init__(
                    self,
                    blue=_zivid.Settings().Processing.Color.Balance().Blue().value,
                    green=_zivid.Settings().Processing.Color.Balance().Green().value,
                    red=_zivid.Settings().Processing.Color.Balance().Red().value,
                ):

                    if isinstance(blue, (float, int,)) or blue is None:
                        self._blue = _zivid.Settings.Processing.Color.Balance.Blue(blue)
                    else:
                        raise TypeError(
                            "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                                value_type=type(blue)
                            )
                        )
                    if isinstance(green, (float, int,)) or green is None:
                        self._green = _zivid.Settings.Processing.Color.Balance.Green(
                            green
                        )
                    else:
                        raise TypeError(
                            "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                                value_type=type(green)
                            )
                        )
                    if isinstance(red, (float, int,)) or red is None:
                        self._red = _zivid.Settings.Processing.Color.Balance.Red(red)
                    else:
                        raise TypeError(
                            "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                                value_type=type(red)
                            )
                        )

                @property
                def blue(self):
                    return self._blue.value

                @property
                def green(self):
                    return self._green.value

                @property
                def red(self):
                    return self._red.value

                @blue.setter
                def blue(self, value):
                    if isinstance(value, (float, int,)) or value is None:
                        self._blue = _zivid.Settings.Processing.Color.Balance.Blue(
                            value
                        )
                    else:
                        raise TypeError(
                            "Unsupported type, expected: float or  int or None, got {value_type}".format(
                                value_type=type(value)
                            )
                        )

                @green.setter
                def green(self, value):
                    if isinstance(value, (float, int,)) or value is None:
                        self._green = _zivid.Settings.Processing.Color.Balance.Green(
                            value
                        )
                    else:
                        raise TypeError(
                            "Unsupported type, expected: float or  int or None, got {value_type}".format(
                                value_type=type(value)
                            )
                        )

                @red.setter
                def red(self, value):
                    if isinstance(value, (float, int,)) or value is None:
                        self._red = _zivid.Settings.Processing.Color.Balance.Red(value)
                    else:
                        raise TypeError(
                            "Unsupported type, expected: float or  int or None, got {value_type}".format(
                                value_type=type(value)
                            )
                        )

                def __eq__(self, other):
                    if (
                        self._blue == other._blue
                        and self._green == other._green
                        and self._red == other._red
                    ):
                        return True
                    return False

                def __str__(self):
                    return str(
                        zivid._settings_converter.to_internal_settings_processing_color_balance(
                            self
                        )
                    )

            def __init__(
                self,
                gamma=_zivid.Settings().Processing.Color().Gamma().value,
                balance=None,
            ):

                if isinstance(gamma, (float, int,)) or gamma is None:
                    self._gamma = _zivid.Settings.Processing.Color.Gamma(gamma)
                else:
                    raise TypeError(
                        "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                            value_type=type(gamma)
                        )
                    )
                if balance is None:
                    balance = zivid.Settings.Processing.Color.Balance()
                if not isinstance(balance, zivid.Settings.Processing.Color.Balance):
                    raise TypeError(
                        "Unsupported type: {value}".format(value=type(balance))
                    )
                self._balance = balance

            @property
            def gamma(self):
                return self._gamma.value

            @property
            def balance(self):
                return self._balance

            @gamma.setter
            def gamma(self, value):
                if isinstance(value, (float, int,)) or value is None:
                    self._gamma = _zivid.Settings.Processing.Color.Gamma(value)
                else:
                    raise TypeError(
                        "Unsupported type, expected: float or  int or None, got {value_type}".format(
                            value_type=type(value)
                        )
                    )

            @balance.setter
            def balance(self, value):
                if not isinstance(value, zivid.Settings.Processing.Color.Balance):
                    raise TypeError(
                        "Unsupported type {value}".format(value=type(value))
                    )
                self._balance = value

            def __eq__(self, other):
                if self._gamma == other._gamma and self._balance == other._balance:
                    return True
                return False

            def __str__(self):
                return str(
                    zivid._settings_converter.to_internal_settings_processing_color(
                        self
                    )
                )

        class Filters:
            class Experimental:
                class ContrastDistortion:
                    class Correction:
                        def __init__(
                            self,
                            enabled=_zivid.Settings()
                            .Processing.Filters.Experimental.ContrastDistortion.Correction()
                            .Enabled()
                            .value,
                            strength=_zivid.Settings()
                            .Processing.Filters.Experimental.ContrastDistortion.Correction()
                            .Strength()
                            .value,
                        ):

                            if isinstance(enabled, (bool,)) or enabled is None:
                                self._enabled = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction.Enabled(
                                    enabled
                                )
                            else:
                                raise TypeError(
                                    "Unsupported type, expected: (bool,) or None, got {value_type}".format(
                                        value_type=type(enabled)
                                    )
                                )
                            if isinstance(strength, (float, int,)) or strength is None:
                                self._strength = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction.Strength(
                                    strength
                                )
                            else:
                                raise TypeError(
                                    "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                                        value_type=type(strength)
                                    )
                                )

                        @property
                        def enabled(self):
                            return self._enabled.value

                        @property
                        def strength(self):
                            return self._strength.value

                        @enabled.setter
                        def enabled(self, value):
                            if isinstance(value, (bool,)) or value is None:
                                self._enabled = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction.Enabled(
                                    value
                                )
                            else:
                                raise TypeError(
                                    "Unsupported type, expected: bool or None, got {value_type}".format(
                                        value_type=type(value)
                                    )
                                )

                        @strength.setter
                        def strength(self, value):
                            if isinstance(value, (float, int,)) or value is None:
                                self._strength = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction.Strength(
                                    value
                                )
                            else:
                                raise TypeError(
                                    "Unsupported type, expected: float or  int or None, got {value_type}".format(
                                        value_type=type(value)
                                    )
                                )

                        def __eq__(self, other):
                            if (
                                self._enabled == other._enabled
                                and self._strength == other._strength
                            ):
                                return True
                            return False

                        def __str__(self):
                            return str(
                                zivid._settings_converter.to_internal_settings_processing_filters_experimental_contrast_distortion_correction(
                                    self
                                )
                            )

                    class Removal:
                        def __init__(
                            self,
                            enabled=_zivid.Settings()
                            .Processing.Filters.Experimental.ContrastDistortion.Removal()
                            .Enabled()
                            .value,
                            threshold=_zivid.Settings()
                            .Processing.Filters.Experimental.ContrastDistortion.Removal()
                            .Threshold()
                            .value,
                        ):

                            if isinstance(enabled, (bool,)) or enabled is None:
                                self._enabled = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal.Enabled(
                                    enabled
                                )
                            else:
                                raise TypeError(
                                    "Unsupported type, expected: (bool,) or None, got {value_type}".format(
                                        value_type=type(enabled)
                                    )
                                )
                            if (
                                isinstance(threshold, (float, int,))
                                or threshold is None
                            ):
                                self._threshold = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal.Threshold(
                                    threshold
                                )
                            else:
                                raise TypeError(
                                    "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                                        value_type=type(threshold)
                                    )
                                )

                        @property
                        def enabled(self):
                            return self._enabled.value

                        @property
                        def threshold(self):
                            return self._threshold.value

                        @enabled.setter
                        def enabled(self, value):
                            if isinstance(value, (bool,)) or value is None:
                                self._enabled = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal.Enabled(
                                    value
                                )
                            else:
                                raise TypeError(
                                    "Unsupported type, expected: bool or None, got {value_type}".format(
                                        value_type=type(value)
                                    )
                                )

                        @threshold.setter
                        def threshold(self, value):
                            if isinstance(value, (float, int,)) or value is None:
                                self._threshold = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal.Threshold(
                                    value
                                )
                            else:
                                raise TypeError(
                                    "Unsupported type, expected: float or  int or None, got {value_type}".format(
                                        value_type=type(value)
                                    )
                                )

                        def __eq__(self, other):
                            if (
                                self._enabled == other._enabled
                                and self._threshold == other._threshold
                            ):
                                return True
                            return False

                        def __str__(self):
                            return str(
                                zivid._settings_converter.to_internal_settings_processing_filters_experimental_contrast_distortion_removal(
                                    self
                                )
                            )

                    def __init__(
                        self, correction=None, removal=None,
                    ):

                        if correction is None:
                            correction = (
                                zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction()
                            )
                        if not isinstance(
                            correction,
                            zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction,
                        ):
                            raise TypeError(
                                "Unsupported type: {value}".format(
                                    value=type(correction)
                                )
                            )
                        self._correction = correction
                        if removal is None:
                            removal = (
                                zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal()
                            )
                        if not isinstance(
                            removal,
                            zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal,
                        ):
                            raise TypeError(
                                "Unsupported type: {value}".format(value=type(removal))
                            )
                        self._removal = removal

                    @property
                    def correction(self):
                        return self._correction

                    @property
                    def removal(self):
                        return self._removal

                    @correction.setter
                    def correction(self, value):
                        if not isinstance(
                            value,
                            zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction,
                        ):
                            raise TypeError(
                                "Unsupported type {value}".format(value=type(value))
                            )
                        self._correction = value

                    @removal.setter
                    def removal(self, value):
                        if not isinstance(
                            value,
                            zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal,
                        ):
                            raise TypeError(
                                "Unsupported type {value}".format(value=type(value))
                            )
                        self._removal = value

                    def __eq__(self, other):
                        if (
                            self._correction == other._correction
                            and self._removal == other._removal
                        ):
                            return True
                        return False

                    def __str__(self):
                        return str(
                            zivid._settings_converter.to_internal_settings_processing_filters_experimental_contrast_distortion(
                                self
                            )
                        )

                def __init__(
                    self, contrast_distortion=None,
                ):

                    if contrast_distortion is None:
                        contrast_distortion = (
                            zivid.Settings.Processing.Filters.Experimental.ContrastDistortion()
                        )
                    if not isinstance(
                        contrast_distortion,
                        zivid.Settings.Processing.Filters.Experimental.ContrastDistortion,
                    ):
                        raise TypeError(
                            "Unsupported type: {value}".format(
                                value=type(contrast_distortion)
                            )
                        )
                    self._contrast_distortion = contrast_distortion

                @property
                def contrast_distortion(self):
                    return self._contrast_distortion

                @contrast_distortion.setter
                def contrast_distortion(self, value):
                    if not isinstance(
                        value,
                        zivid.Settings.Processing.Filters.Experimental.ContrastDistortion,
                    ):
                        raise TypeError(
                            "Unsupported type {value}".format(value=type(value))
                        )
                    self._contrast_distortion = value

                def __eq__(self, other):
                    if self._contrast_distortion == other._contrast_distortion:
                        return True
                    return False

                def __str__(self):
                    return str(
                        zivid._settings_converter.to_internal_settings_processing_filters_experimental(
                            self
                        )
                    )

            class Noise:
                class Removal:
                    def __init__(
                        self,
                        enabled=_zivid.Settings()
                        .Processing.Filters.Noise.Removal()
                        .Enabled()
                        .value,
                        threshold=_zivid.Settings()
                        .Processing.Filters.Noise.Removal()
                        .Threshold()
                        .value,
                    ):

                        if isinstance(enabled, (bool,)) or enabled is None:
                            self._enabled = _zivid.Settings.Processing.Filters.Noise.Removal.Enabled(
                                enabled
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (bool,) or None, got {value_type}".format(
                                    value_type=type(enabled)
                                )
                            )
                        if isinstance(threshold, (float, int,)) or threshold is None:
                            self._threshold = _zivid.Settings.Processing.Filters.Noise.Removal.Threshold(
                                threshold
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                                    value_type=type(threshold)
                                )
                            )

                    @property
                    def enabled(self):
                        return self._enabled.value

                    @property
                    def threshold(self):
                        return self._threshold.value

                    @enabled.setter
                    def enabled(self, value):
                        if isinstance(value, (bool,)) or value is None:
                            self._enabled = _zivid.Settings.Processing.Filters.Noise.Removal.Enabled(
                                value
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: bool or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    @threshold.setter
                    def threshold(self, value):
                        if isinstance(value, (float, int,)) or value is None:
                            self._threshold = _zivid.Settings.Processing.Filters.Noise.Removal.Threshold(
                                value
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: float or  int or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    def __eq__(self, other):
                        if (
                            self._enabled == other._enabled
                            and self._threshold == other._threshold
                        ):
                            return True
                        return False

                    def __str__(self):
                        return str(
                            zivid._settings_converter.to_internal_settings_processing_filters_noise_removal(
                                self
                            )
                        )

                def __init__(
                    self, removal=None,
                ):

                    if removal is None:
                        removal = zivid.Settings.Processing.Filters.Noise.Removal()
                    if not isinstance(
                        removal, zivid.Settings.Processing.Filters.Noise.Removal
                    ):
                        raise TypeError(
                            "Unsupported type: {value}".format(value=type(removal))
                        )
                    self._removal = removal

                @property
                def removal(self):
                    return self._removal

                @removal.setter
                def removal(self, value):
                    if not isinstance(
                        value, zivid.Settings.Processing.Filters.Noise.Removal
                    ):
                        raise TypeError(
                            "Unsupported type {value}".format(value=type(value))
                        )
                    self._removal = value

                def __eq__(self, other):
                    if self._removal == other._removal:
                        return True
                    return False

                def __str__(self):
                    return str(
                        zivid._settings_converter.to_internal_settings_processing_filters_noise(
                            self
                        )
                    )

            class Outlier:
                class Removal:
                    def __init__(
                        self,
                        enabled=_zivid.Settings()
                        .Processing.Filters.Outlier.Removal()
                        .Enabled()
                        .value,
                        threshold=_zivid.Settings()
                        .Processing.Filters.Outlier.Removal()
                        .Threshold()
                        .value,
                    ):

                        if isinstance(enabled, (bool,)) or enabled is None:
                            self._enabled = _zivid.Settings.Processing.Filters.Outlier.Removal.Enabled(
                                enabled
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (bool,) or None, got {value_type}".format(
                                    value_type=type(enabled)
                                )
                            )
                        if isinstance(threshold, (float, int,)) or threshold is None:
                            self._threshold = _zivid.Settings.Processing.Filters.Outlier.Removal.Threshold(
                                threshold
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                                    value_type=type(threshold)
                                )
                            )

                    @property
                    def enabled(self):
                        return self._enabled.value

                    @property
                    def threshold(self):
                        return self._threshold.value

                    @enabled.setter
                    def enabled(self, value):
                        if isinstance(value, (bool,)) or value is None:
                            self._enabled = _zivid.Settings.Processing.Filters.Outlier.Removal.Enabled(
                                value
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: bool or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    @threshold.setter
                    def threshold(self, value):
                        if isinstance(value, (float, int,)) or value is None:
                            self._threshold = _zivid.Settings.Processing.Filters.Outlier.Removal.Threshold(
                                value
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: float or  int or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    def __eq__(self, other):
                        if (
                            self._enabled == other._enabled
                            and self._threshold == other._threshold
                        ):
                            return True
                        return False

                    def __str__(self):
                        return str(
                            zivid._settings_converter.to_internal_settings_processing_filters_outlier_removal(
                                self
                            )
                        )

                def __init__(
                    self, removal=None,
                ):

                    if removal is None:
                        removal = zivid.Settings.Processing.Filters.Outlier.Removal()
                    if not isinstance(
                        removal, zivid.Settings.Processing.Filters.Outlier.Removal
                    ):
                        raise TypeError(
                            "Unsupported type: {value}".format(value=type(removal))
                        )
                    self._removal = removal

                @property
                def removal(self):
                    return self._removal

                @removal.setter
                def removal(self, value):
                    if not isinstance(
                        value, zivid.Settings.Processing.Filters.Outlier.Removal
                    ):
                        raise TypeError(
                            "Unsupported type {value}".format(value=type(value))
                        )
                    self._removal = value

                def __eq__(self, other):
                    if self._removal == other._removal:
                        return True
                    return False

                def __str__(self):
                    return str(
                        zivid._settings_converter.to_internal_settings_processing_filters_outlier(
                            self
                        )
                    )

            class Reflection:
                class Removal:
                    def __init__(
                        self,
                        enabled=_zivid.Settings()
                        .Processing.Filters.Reflection.Removal()
                        .Enabled()
                        .value,
                    ):

                        if isinstance(enabled, (bool,)) or enabled is None:
                            self._enabled = _zivid.Settings.Processing.Filters.Reflection.Removal.Enabled(
                                enabled
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (bool,) or None, got {value_type}".format(
                                    value_type=type(enabled)
                                )
                            )

                    @property
                    def enabled(self):
                        return self._enabled.value

                    @enabled.setter
                    def enabled(self, value):
                        if isinstance(value, (bool,)) or value is None:
                            self._enabled = _zivid.Settings.Processing.Filters.Reflection.Removal.Enabled(
                                value
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: bool or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    def __eq__(self, other):
                        if self._enabled == other._enabled:
                            return True
                        return False

                    def __str__(self):
                        return str(
                            zivid._settings_converter.to_internal_settings_processing_filters_reflection_removal(
                                self
                            )
                        )

                def __init__(
                    self, removal=None,
                ):

                    if removal is None:
                        removal = zivid.Settings.Processing.Filters.Reflection.Removal()
                    if not isinstance(
                        removal, zivid.Settings.Processing.Filters.Reflection.Removal
                    ):
                        raise TypeError(
                            "Unsupported type: {value}".format(value=type(removal))
                        )
                    self._removal = removal

                @property
                def removal(self):
                    return self._removal

                @removal.setter
                def removal(self, value):
                    if not isinstance(
                        value, zivid.Settings.Processing.Filters.Reflection.Removal
                    ):
                        raise TypeError(
                            "Unsupported type {value}".format(value=type(value))
                        )
                    self._removal = value

                def __eq__(self, other):
                    if self._removal == other._removal:
                        return True
                    return False

                def __str__(self):
                    return str(
                        zivid._settings_converter.to_internal_settings_processing_filters_reflection(
                            self
                        )
                    )

            class Smoothing:
                class Gaussian:
                    def __init__(
                        self,
                        enabled=_zivid.Settings()
                        .Processing.Filters.Smoothing.Gaussian()
                        .Enabled()
                        .value,
                        sigma=_zivid.Settings()
                        .Processing.Filters.Smoothing.Gaussian()
                        .Sigma()
                        .value,
                    ):

                        if isinstance(enabled, (bool,)) or enabled is None:
                            self._enabled = _zivid.Settings.Processing.Filters.Smoothing.Gaussian.Enabled(
                                enabled
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (bool,) or None, got {value_type}".format(
                                    value_type=type(enabled)
                                )
                            )
                        if isinstance(sigma, (float, int,)) or sigma is None:
                            self._sigma = _zivid.Settings.Processing.Filters.Smoothing.Gaussian.Sigma(
                                sigma
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                                    value_type=type(sigma)
                                )
                            )

                    @property
                    def enabled(self):
                        return self._enabled.value

                    @property
                    def sigma(self):
                        return self._sigma.value

                    @enabled.setter
                    def enabled(self, value):
                        if isinstance(value, (bool,)) or value is None:
                            self._enabled = _zivid.Settings.Processing.Filters.Smoothing.Gaussian.Enabled(
                                value
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: bool or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    @sigma.setter
                    def sigma(self, value):
                        if isinstance(value, (float, int,)) or value is None:
                            self._sigma = _zivid.Settings.Processing.Filters.Smoothing.Gaussian.Sigma(
                                value
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: float or  int or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    def __eq__(self, other):
                        if (
                            self._enabled == other._enabled
                            and self._sigma == other._sigma
                        ):
                            return True
                        return False

                    def __str__(self):
                        return str(
                            zivid._settings_converter.to_internal_settings_processing_filters_smoothing_gaussian(
                                self
                            )
                        )

                def __init__(
                    self, gaussian=None,
                ):

                    if gaussian is None:
                        gaussian = (
                            zivid.Settings.Processing.Filters.Smoothing.Gaussian()
                        )
                    if not isinstance(
                        gaussian, zivid.Settings.Processing.Filters.Smoothing.Gaussian
                    ):
                        raise TypeError(
                            "Unsupported type: {value}".format(value=type(gaussian))
                        )
                    self._gaussian = gaussian

                @property
                def gaussian(self):
                    return self._gaussian

                @gaussian.setter
                def gaussian(self, value):
                    if not isinstance(
                        value, zivid.Settings.Processing.Filters.Smoothing.Gaussian
                    ):
                        raise TypeError(
                            "Unsupported type {value}".format(value=type(value))
                        )
                    self._gaussian = value

                def __eq__(self, other):
                    if self._gaussian == other._gaussian:
                        return True
                    return False

                def __str__(self):
                    return str(
                        zivid._settings_converter.to_internal_settings_processing_filters_smoothing(
                            self
                        )
                    )

            def __init__(
                self,
                experimental=None,
                noise=None,
                outlier=None,
                reflection=None,
                smoothing=None,
            ):

                if experimental is None:
                    experimental = zivid.Settings.Processing.Filters.Experimental()
                if not isinstance(
                    experimental, zivid.Settings.Processing.Filters.Experimental
                ):
                    raise TypeError(
                        "Unsupported type: {value}".format(value=type(experimental))
                    )
                self._experimental = experimental
                if noise is None:
                    noise = zivid.Settings.Processing.Filters.Noise()
                if not isinstance(noise, zivid.Settings.Processing.Filters.Noise):
                    raise TypeError(
                        "Unsupported type: {value}".format(value=type(noise))
                    )
                self._noise = noise
                if outlier is None:
                    outlier = zivid.Settings.Processing.Filters.Outlier()
                if not isinstance(outlier, zivid.Settings.Processing.Filters.Outlier):
                    raise TypeError(
                        "Unsupported type: {value}".format(value=type(outlier))
                    )
                self._outlier = outlier
                if reflection is None:
                    reflection = zivid.Settings.Processing.Filters.Reflection()
                if not isinstance(
                    reflection, zivid.Settings.Processing.Filters.Reflection
                ):
                    raise TypeError(
                        "Unsupported type: {value}".format(value=type(reflection))
                    )
                self._reflection = reflection
                if smoothing is None:
                    smoothing = zivid.Settings.Processing.Filters.Smoothing()
                if not isinstance(
                    smoothing, zivid.Settings.Processing.Filters.Smoothing
                ):
                    raise TypeError(
                        "Unsupported type: {value}".format(value=type(smoothing))
                    )
                self._smoothing = smoothing

            @property
            def experimental(self):
                return self._experimental

            @property
            def noise(self):
                return self._noise

            @property
            def outlier(self):
                return self._outlier

            @property
            def reflection(self):
                return self._reflection

            @property
            def smoothing(self):
                return self._smoothing

            @experimental.setter
            def experimental(self, value):
                if not isinstance(
                    value, zivid.Settings.Processing.Filters.Experimental
                ):
                    raise TypeError(
                        "Unsupported type {value}".format(value=type(value))
                    )
                self._experimental = value

            @noise.setter
            def noise(self, value):
                if not isinstance(value, zivid.Settings.Processing.Filters.Noise):
                    raise TypeError(
                        "Unsupported type {value}".format(value=type(value))
                    )
                self._noise = value

            @outlier.setter
            def outlier(self, value):
                if not isinstance(value, zivid.Settings.Processing.Filters.Outlier):
                    raise TypeError(
                        "Unsupported type {value}".format(value=type(value))
                    )
                self._outlier = value

            @reflection.setter
            def reflection(self, value):
                if not isinstance(value, zivid.Settings.Processing.Filters.Reflection):
                    raise TypeError(
                        "Unsupported type {value}".format(value=type(value))
                    )
                self._reflection = value

            @smoothing.setter
            def smoothing(self, value):
                if not isinstance(value, zivid.Settings.Processing.Filters.Smoothing):
                    raise TypeError(
                        "Unsupported type {value}".format(value=type(value))
                    )
                self._smoothing = value

            def __eq__(self, other):
                if (
                    self._experimental == other._experimental
                    and self._noise == other._noise
                    and self._outlier == other._outlier
                    and self._reflection == other._reflection
                    and self._smoothing == other._smoothing
                ):
                    return True
                return False

            def __str__(self):
                return str(
                    zivid._settings_converter.to_internal_settings_processing_filters(
                        self
                    )
                )

        def __init__(
            self, color=None, filters=None,
        ):

            if color is None:
                color = zivid.Settings.Processing.Color()
            if not isinstance(color, zivid.Settings.Processing.Color):
                raise TypeError("Unsupported type: {value}".format(value=type(color)))
            self._color = color
            if filters is None:
                filters = zivid.Settings.Processing.Filters()
            if not isinstance(filters, zivid.Settings.Processing.Filters):
                raise TypeError("Unsupported type: {value}".format(value=type(filters)))
            self._filters = filters

        @property
        def color(self):
            return self._color

        @property
        def filters(self):
            return self._filters

        @color.setter
        def color(self, value):
            if not isinstance(value, zivid.Settings.Processing.Color):
                raise TypeError("Unsupported type {value}".format(value=type(value)))
            self._color = value

        @filters.setter
        def filters(self, value):
            if not isinstance(value, zivid.Settings.Processing.Filters):
                raise TypeError("Unsupported type {value}".format(value=type(value)))
            self._filters = value

        def __eq__(self, other):
            if self._color == other._color and self._filters == other._filters:
                return True
            return False

        def __str__(self):
            return str(zivid._settings_converter.to_internal_settings_processing(self))

    def __str__(self):
        return str(zivid._settings_converter.to_internal_settings(self))

    @property
    def processing(self):
        return self._processing

    @processing.setter
    def processing(self, value):
        if not isinstance(value, zivid.Settings.Processing):
            raise TypeError("Unsupported type {value}".format(value=type(value)))
        self._processing = value

    def __init__(
        self, acquisitions=None, processing=None,
    ):
        if acquisitions is None:
            acquisitions = _zivid.Settings().Acquisitions().value
        if not isinstance(acquisitions, collections.abc.Iterable):
            raise TypeError(
                "Unsupported type: {value}".format(value=type(acquisitions))
            )
        self._acquisitions = _convert_to_acquistions(acquisitions)

        if processing is None:
            processing = zivid.Settings.Processing()
        if not isinstance(processing, zivid.Settings.Processing):
            raise TypeError("Unsupported type: {value}".format(value=type(processing)))
        self._processing = processing

    def __eq__(self, other):
        if (
            self._acquisitions == other._acquisitions
            and self._processing == other._processing
        ):
            return True
        return False

    @property
    def acquisitions(self):
        return self._acquisitions

    @acquisitions.setter
    def acquisitions(self, value):
        if not isinstance(value, collections.abc.Iterable):
            raise TypeError("Unsupported type: {value}".format(value=type(value)))
        self._acquisitions = _convert_to_acquistions(value)


def _convert_to_acquistions(inputs):
    temp = []
    for acquisition_element in inputs:
        if isinstance(acquisition_element, Settings.Acquisition):
            temp.append(acquisition_element)
        else:
            raise TypeError(
                "Unsupported type {type_of_acquisition_element}".format(
                    type_of_acquisition_element=type(acquisition_element)
                )
            )
    return temp



"""Auto generated, do not edit."""
import zivid
import _zivid


def to_camera_state_temperature(internal_temperature):
    return zivid.CameraState.Temperature(
        dmd=internal_temperature.dmd.value,
        general=internal_temperature.general.value,
        led=internal_temperature.led.value,
        lens=internal_temperature.lens.value,
        pcb=internal_temperature.pcb.value,
    )


def to_camera_state(internal_camera_state):
    return zivid.CameraState(
        temperature=to_camera_state_temperature(internal_camera_state.temperature),
        available=internal_camera_state.available.value,
        connected=internal_camera_state.connected.value,
    )


def to_internal_camera_state_temperature(temperature):
    internal_temperature = _zivid.CameraState.Temperature()

    internal_temperature.dmd = _zivid.CameraState.Temperature.DMD(temperature.dmd)
    internal_temperature.general = _zivid.CameraState.Temperature.General(
        temperature.general
    )
    internal_temperature.led = _zivid.CameraState.Temperature.LED(temperature.led)
    internal_temperature.lens = _zivid.CameraState.Temperature.Lens(temperature.lens)
    internal_temperature.pcb = _zivid.CameraState.Temperature.PCB(temperature.pcb)

    return internal_temperature


def to_internal_camera_state(camera_state):
    internal_camera_state = _zivid.CameraState()

    internal_camera_state.available = _zivid.CameraState.Available(
        camera_state.available
    )
    internal_camera_state.connected = _zivid.CameraState.Connected(
        camera_state.connected
    )

    internal_camera_state.temperature = to_internal_camera_state_temperature(
        camera_state.temperature
    )
    return internal_camera_state



"""Auto generated, do not edit."""
import zivid
import _zivid


def to_settings_acquisition(internal_acquisition):
    return zivid.Settings.Acquisition(
        aperture=internal_acquisition.aperture.value,
        brightness=internal_acquisition.brightness.value,
        exposure_time=internal_acquisition.exposure_time.value,
        gain=internal_acquisition.gain.value,
    )


def to_settings_processing_color_balance(internal_balance):
    return zivid.Settings.Processing.Color.Balance(
        blue=internal_balance.blue.value,
        green=internal_balance.green.value,
        red=internal_balance.red.value,
    )


def to_settings_processing_color(internal_color):
    return zivid.Settings.Processing.Color(
        balance=to_settings_processing_color_balance(internal_color.balance),
        gamma=internal_color.gamma.value,
    )


def to_settings_processing_filters_experimental_contrast_distortion_correction(
    internal_correction,
):
    return zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction(
        enabled=internal_correction.enabled.value,
        strength=internal_correction.strength.value,
    )


def to_settings_processing_filters_experimental_contrast_distortion_removal(
    internal_removal,
):
    return zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal(
        enabled=internal_removal.enabled.value,
        threshold=internal_removal.threshold.value,
    )


def to_settings_processing_filters_experimental_contrast_distortion(
    internal_contrast_distortion,
):
    return zivid.Settings.Processing.Filters.Experimental.ContrastDistortion(
        correction=to_settings_processing_filters_experimental_contrast_distortion_correction(
            internal_contrast_distortion.correction
        ),
        removal=to_settings_processing_filters_experimental_contrast_distortion_removal(
            internal_contrast_distortion.removal
        ),
    )


def to_settings_processing_filters_experimental(internal_experimental):
    return zivid.Settings.Processing.Filters.Experimental(
        contrast_distortion=to_settings_processing_filters_experimental_contrast_distortion(
            internal_experimental.contrast_distortion
        ),
    )


def to_settings_processing_filters_noise_removal(internal_removal):
    return zivid.Settings.Processing.Filters.Noise.Removal(
        enabled=internal_removal.enabled.value,
        threshold=internal_removal.threshold.value,
    )


def to_settings_processing_filters_noise(internal_noise):
    return zivid.Settings.Processing.Filters.Noise(
        removal=to_settings_processing_filters_noise_removal(internal_noise.removal),
    )


def to_settings_processing_filters_outlier_removal(internal_removal):
    return zivid.Settings.Processing.Filters.Outlier.Removal(
        enabled=internal_removal.enabled.value,
        threshold=internal_removal.threshold.value,
    )


def to_settings_processing_filters_outlier(internal_outlier):
    return zivid.Settings.Processing.Filters.Outlier(
        removal=to_settings_processing_filters_outlier_removal(
            internal_outlier.removal
        ),
    )


def to_settings_processing_filters_reflection_removal(internal_removal):
    return zivid.Settings.Processing.Filters.Reflection.Removal(
        enabled=internal_removal.enabled.value,
    )


def to_settings_processing_filters_reflection(internal_reflection):
    return zivid.Settings.Processing.Filters.Reflection(
        removal=to_settings_processing_filters_reflection_removal(
            internal_reflection.removal
        ),
    )


def to_settings_processing_filters_smoothing_gaussian(internal_gaussian):
    return zivid.Settings.Processing.Filters.Smoothing.Gaussian(
        enabled=internal_gaussian.enabled.value, sigma=internal_gaussian.sigma.value,
    )


def to_settings_processing_filters_smoothing(internal_smoothing):
    return zivid.Settings.Processing.Filters.Smoothing(
        gaussian=to_settings_processing_filters_smoothing_gaussian(
            internal_smoothing.gaussian
        ),
    )


def to_settings_processing_filters(internal_filters):
    return zivid.Settings.Processing.Filters(
        experimental=to_settings_processing_filters_experimental(
            internal_filters.experimental
        ),
        noise=to_settings_processing_filters_noise(internal_filters.noise),
        outlier=to_settings_processing_filters_outlier(internal_filters.outlier),
        reflection=to_settings_processing_filters_reflection(
            internal_filters.reflection
        ),
        smoothing=to_settings_processing_filters_smoothing(internal_filters.smoothing),
    )


def to_settings_processing(internal_processing):
    return zivid.Settings.Processing(
        color=to_settings_processing_color(internal_processing.color),
        filters=to_settings_processing_filters(internal_processing.filters),
    )


def to_settings(internal_settings):
    return zivid.Settings(
        processing=to_settings_processing(internal_settings.processing),
        acquisitions=[
            to_settings_acquisition(element)
            for element in internal_settings.acquisitions.value
        ],
    )


def to_internal_settings_acquisition(acquisition):
    internal_acquisition = _zivid.Settings.Acquisition()

    internal_acquisition.aperture = _zivid.Settings.Acquisition.Aperture(
        acquisition.aperture
    )
    internal_acquisition.brightness = _zivid.Settings.Acquisition.Brightness(
        acquisition.brightness
    )
    internal_acquisition.exposure_time = _zivid.Settings.Acquisition.ExposureTime(
        acquisition.exposure_time
    )
    internal_acquisition.gain = _zivid.Settings.Acquisition.Gain(acquisition.gain)

    return internal_acquisition


def to_internal_settings_processing_color_balance(balance):
    internal_balance = _zivid.Settings.Processing.Color.Balance()

    internal_balance.blue = _zivid.Settings.Processing.Color.Balance.Blue(balance.blue)
    internal_balance.green = _zivid.Settings.Processing.Color.Balance.Green(
        balance.green
    )
    internal_balance.red = _zivid.Settings.Processing.Color.Balance.Red(balance.red)

    return internal_balance


def to_internal_settings_processing_color(color):
    internal_color = _zivid.Settings.Processing.Color()

    internal_color.gamma = _zivid.Settings.Processing.Color.Gamma(color.gamma)
    internal_color.balance = to_internal_settings_processing_color_balance(
        color.balance
    )
    return internal_color


def to_internal_settings_processing_filters_experimental_contrast_distortion_correction(
    correction,
):
    internal_correction = (
        _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction()
    )

    internal_correction.enabled = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction.Enabled(
        correction.enabled
    )
    internal_correction.strength = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction.Strength(
        correction.strength
    )

    return internal_correction


def to_internal_settings_processing_filters_experimental_contrast_distortion_removal(
    removal,
):
    internal_removal = (
        _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal()
    )

    internal_removal.enabled = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal.Enabled(
        removal.enabled
    )
    internal_removal.threshold = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal.Threshold(
        removal.threshold
    )

    return internal_removal


def to_internal_settings_processing_filters_experimental_contrast_distortion(
    contrast_distortion,
):
    internal_contrast_distortion = (
        _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion()
    )

    internal_contrast_distortion.correction = to_internal_settings_processing_filters_experimental_contrast_distortion_correction(
        contrast_distortion.correction
    )
    internal_contrast_distortion.removal = to_internal_settings_processing_filters_experimental_contrast_distortion_removal(
        contrast_distortion.removal
    )
    return internal_contrast_distortion


def to_internal_settings_processing_filters_experimental(experimental):
    internal_experimental = _zivid.Settings.Processing.Filters.Experimental()

    internal_experimental.contrast_distortion = to_internal_settings_processing_filters_experimental_contrast_distortion(
        experimental.contrast_distortion
    )
    return internal_experimental


def to_internal_settings_processing_filters_noise_removal(removal):
    internal_removal = _zivid.Settings.Processing.Filters.Noise.Removal()

    internal_removal.enabled = _zivid.Settings.Processing.Filters.Noise.Removal.Enabled(
        removal.enabled
    )
    internal_removal.threshold = _zivid.Settings.Processing.Filters.Noise.Removal.Threshold(
        removal.threshold
    )

    return internal_removal


def to_internal_settings_processing_filters_noise(noise):
    internal_noise = _zivid.Settings.Processing.Filters.Noise()

    internal_noise.removal = to_internal_settings_processing_filters_noise_removal(
        noise.removal
    )
    return internal_noise


def to_internal_settings_processing_filters_outlier_removal(removal):
    internal_removal = _zivid.Settings.Processing.Filters.Outlier.Removal()

    internal_removal.enabled = _zivid.Settings.Processing.Filters.Outlier.Removal.Enabled(
        removal.enabled
    )
    internal_removal.threshold = _zivid.Settings.Processing.Filters.Outlier.Removal.Threshold(
        removal.threshold
    )

    return internal_removal


def to_internal_settings_processing_filters_outlier(outlier):
    internal_outlier = _zivid.Settings.Processing.Filters.Outlier()

    internal_outlier.removal = to_internal_settings_processing_filters_outlier_removal(
        outlier.removal
    )
    return internal_outlier


def to_internal_settings_processing_filters_reflection_removal(removal):
    internal_removal = _zivid.Settings.Processing.Filters.Reflection.Removal()

    internal_removal.enabled = _zivid.Settings.Processing.Filters.Reflection.Removal.Enabled(
        removal.enabled
    )

    return internal_removal


def to_internal_settings_processing_filters_reflection(reflection):
    internal_reflection = _zivid.Settings.Processing.Filters.Reflection()

    internal_reflection.removal = to_internal_settings_processing_filters_reflection_removal(
        reflection.removal
    )
    return internal_reflection


def to_internal_settings_processing_filters_smoothing_gaussian(gaussian):
    internal_gaussian = _zivid.Settings.Processing.Filters.Smoothing.Gaussian()

    internal_gaussian.enabled = _zivid.Settings.Processing.Filters.Smoothing.Gaussian.Enabled(
        gaussian.enabled
    )
    internal_gaussian.sigma = _zivid.Settings.Processing.Filters.Smoothing.Gaussian.Sigma(
        gaussian.sigma
    )

    return internal_gaussian


def to_internal_settings_processing_filters_smoothing(smoothing):
    internal_smoothing = _zivid.Settings.Processing.Filters.Smoothing()

    internal_smoothing.gaussian = to_internal_settings_processing_filters_smoothing_gaussian(
        smoothing.gaussian
    )
    return internal_smoothing


def to_internal_settings_processing_filters(filters):
    internal_filters = _zivid.Settings.Processing.Filters()

    internal_filters.experimental = to_internal_settings_processing_filters_experimental(
        filters.experimental
    )
    internal_filters.noise = to_internal_settings_processing_filters_noise(
        filters.noise
    )
    internal_filters.outlier = to_internal_settings_processing_filters_outlier(
        filters.outlier
    )
    internal_filters.reflection = to_internal_settings_processing_filters_reflection(
        filters.reflection
    )
    internal_filters.smoothing = to_internal_settings_processing_filters_smoothing(
        filters.smoothing
    )
    return internal_filters


def to_internal_settings_processing(processing):
    internal_processing = _zivid.Settings.Processing()

    internal_processing.color = to_internal_settings_processing_color(processing.color)
    internal_processing.filters = to_internal_settings_processing_filters(
        processing.filters
    )
    return internal_processing


def to_internal_settings(settings):
    internal_settings = _zivid.Settings()

    internal_settings.processing = to_internal_settings_processing(settings.processing)

    temp = _zivid.Settings().Acquisitions()
    for acq in settings.acquisitions:
        temp.append(to_internal_settings_acquisition(acq))
    internal_settings.acquisitions = temp

    return internal_settings



"""Auto generated, do not edit."""
import _zivid
import zivid


def to_capture_assistant_suggest_settings_parameters_ambient_light_frequency(
    internal_ambient_light_frequency,
):
    for (
        key,
        value,
    ) in (
        zivid.capture_assistant.SuggestSettingsParameters.AmbientLightFrequency._valid_values.items()  # pylint: disable=protected-access
    ):
        if value == internal_ambient_light_frequency.value:
            return key
    raise ValueError(
        "Unsupported value: {value}".format(value=internal_ambient_light_frequency)
    )


def to_capture_assistant_suggest_settings_parameters(
    internal_suggest_settings_parameters,
):
    return zivid.capture_assistant.SuggestSettingsParameters(
        ambient_light_frequency=to_capture_assistant_suggest_settings_parameters_ambient_light_frequency(
            internal_suggest_settings_parameters.ambient_light_frequency
        ),
        max_capture_time=internal_suggest_settings_parameters.max_capture_time.value,
    )


def to_internal_capture_assistant_suggest_settings_parameters(
    suggest_settings_parameters,
):
    internal_suggest_settings_parameters = (
        _zivid.capture_assistant.SuggestSettingsParameters()
    )

    internal_suggest_settings_parameters.ambient_light_frequency = to_internal_capture_assistant_suggest_settings_parameters_ambient_light_frequency(
        suggest_settings_parameters.ambient_light_frequency
    )
    internal_suggest_settings_parameters.max_capture_time = _zivid.capture_assistant.SuggestSettingsParameters.MaxCaptureTime(
        suggest_settings_parameters.max_capture_time
    )

    return internal_suggest_settings_parameters


def to_internal_capture_assistant_suggest_settings_parameters_ambient_light_frequency(
    ambient_light_frequency,
):
    try:
        return _zivid.capture_assistant.SuggestSettingsParameters.AmbientLightFrequency(
            zivid.capture_assistant.SuggestSettingsParameters.AmbientLightFrequency._valid_values[  # pylint: disable=protected-access
                ambient_light_frequency
            ]
        )
    except Exception as ex:
        raise ValueError(
            "Unsupported value: {value}".format(value=ambient_light_frequency)
        ) from ex



"""This file imports used classes, modules and packages."""
import inspect
from collections import namedtuple
from dataclasses import dataclass
from typing import Tuple
import subprocess
from _zivid.common import (
    _create_class,
    _imports,
    _recursion,
    create_to_internal_converter,
)
import tempfile
from pathlib import Path
import inflection


def start_traverse():
    from _zivid._zivid import Settings2D as InternalSettings
    from zivid import Settings

    data_model = _recursion(InternalSettings, indentation_level=0)
    with tempfile.NamedTemporaryFile(suffix=".py") as temp_file:
        temp_file = Path(temp_file.name)
        raw_text = _imports(internal=True, settings=False)
        raw_text += create_to_internal_converter(data_model, settings_type="Settings2D")

        new_lines = []
        for line in raw_text.splitlines():
            new_lines.append(line[4:])

        temp_file.write_text("\n".join(new_lines))
        print(temp_file.read_text())
        subprocess.check_output((f"black {temp_file}"), shell=True)
        print(temp_file.read_text())
        path_to_settings = (
            Path(__file__).resolve() / ".." / ".." / "zivid" / "settings__3d.py"
        ).resolve()
        path_to_settings.write_text(temp_file.read_text())



"""Auto generated, do not edit."""
import zivid
import _zivid


def to_camera_info_revision(internal_revision):
    return zivid.CameraInfo.Revision(
        major=internal_revision.major.value, minor=internal_revision.minor.value,
    )


def to_camera_info_user_data(internal_user_data):
    return zivid.CameraInfo.UserData(
        max_size_bytes=internal_user_data.max_size_bytes.value,
    )


def to_camera_info(internal_camera_info):
    return zivid.CameraInfo(
        revision=to_camera_info_revision(internal_camera_info.revision),
        user_data=to_camera_info_user_data(internal_camera_info.user_data),
        firmware_version=internal_camera_info.firmware_version.value,
        model_name=internal_camera_info.model_name.value,
        serial_number=internal_camera_info.serial_number.value,
    )


def to_internal_camera_info_revision(revision):
    internal_revision = _zivid.CameraInfo.Revision()

    internal_revision.major = _zivid.CameraInfo.Revision.Major(revision.major)
    internal_revision.minor = _zivid.CameraInfo.Revision.Minor(revision.minor)

    return internal_revision


def to_internal_camera_info_user_data(user_data):
    internal_user_data = _zivid.CameraInfo.UserData()

    internal_user_data.max_size_bytes = _zivid.CameraInfo.UserData.MaxSizeBytes(
        user_data.max_size_bytes
    )

    return internal_user_data


def to_internal_camera_info(camera_info):
    internal_camera_info = _zivid.CameraInfo()

    internal_camera_info.firmware_version = _zivid.CameraInfo.FirmwareVersion(
        camera_info.firmware_version
    )
    internal_camera_info.model_name = _zivid.CameraInfo.ModelName(
        camera_info.model_name
    )
    internal_camera_info.serial_number = _zivid.CameraInfo.SerialNumber(
        camera_info.serial_number
    )

    internal_camera_info.revision = to_internal_camera_info_revision(
        camera_info.revision
    )
    internal_camera_info.user_data = to_internal_camera_info_user_data(
        camera_info.user_data
    )
    return internal_camera_info



import inspect
import tempfile
from typing import List, Tuple, Optional
from dataclasses import dataclass
import inspect
from collections import namedtuple
from pathlib import Path
import subprocess
import inflection


@dataclass
class MemberVariable:
    camel_case: str
    snake_case: str
    default_value: str
    underlying_type: str
    is_optional: bool


@dataclass
class UnderlyingLeafValue:
    is_optional: bool
    value: str


@dataclass
class NodeData:
    name: str
    is_leaf: bool
    is_enum: bool
    enum_vars: tuple
    enum_default_value: str
    path: str
    children: tuple
    member_variables: Tuple[MemberVariable]
    indentation_level: int
    _zivid_class: str


def _inner_classes_list(cls) -> List:
    return [
        cls_attribute
        for cls_attribute in cls.__dict__.values()
        if inspect.isclass(cls_attribute)
    ]


def _imports(
    internal: bool, settings: bool, additional_imports: tuple = tuple()
) -> str:
    imports = "    '''Auto generated, do not edit'''\n"
    imports += "    import datetime\n"
    imports += "    import types\n"
    imports += "    import collections.abc\n"
    if internal:
        imports += "    import _zivid\n"
    if settings:
        imports += "    import zivid\n"
    for additional_import in additional_imports:
        imports += f"    import {additional_import}"
    return imports


def _get_member_variables(node_data, settings_type: str):
    member_variables = []
    if node_data.member_variables:
        for member_var in node_data.member_variables:
            print(member_var)
            default_value = f"_zivid.{settings_type}().{f'{node_data.path}().' if node_data.path else ''}{member_var.camel_case}().value"
            print("this is current class when getting member variables:")
            print()
            member_variables.append(
                MemberVariable(
                    camel_case=member_var.camel_case,
                    default_value=default_value,
                    snake_case=member_var.snake_case,
                    underlying_type=member_var.underlying_type,
                    is_optional=member_var.is_optional,
                )
            )
    print(member_variables)
    return member_variables


def _get_child_class_member_variables(node_data):
    child_class_members_variables = []
    if node_data.children:
        for child in node_data.children:
            child_class_members_variables.append(
                MemberVariable(
                    camel_case=child.name,
                    snake_case=inflection.underscore(child.name),
                    default_value="None",
                    underlying_type=None,
                    is_optional=True,
                )
            )

    return child_class_members_variables


def _variable_names(node_data, settings_type: str):
    member_variables = _get_member_variables(node_data, settings_type)
    child_class_member_variables = _get_child_class_member_variables(node_data)
    variable_names = list()
    for member in member_variables:
        variable_names.append(member.snake_case)
    for child_class in child_class_member_variables:
        variable_names.append(child_class.snake_case)
    return variable_names


def _create_init_special_member_function(node_data, settings_type: str):
    if node_data.is_enum:
        return "\n    def __init__(self,value=none):\n        self._value = value\n"
    member_variables = _get_member_variables(node_data, settings_type)
    child_class_member_variables = _get_child_class_member_variables(node_data)
    signature_vars = ""
    member_variable_set = ""
    path = ".{path}".format(path=node_data.path,) if node_data.path else ""
    for member in member_variables:
        if member.is_optional:
            is_none_check = f"or {member.snake_case} is None"
            can_be_none_error_message_part = " or None"
        else:
            is_none_check = ""
            can_be_none_error_message_part = ""
        signature_vars += f"{member.snake_case}={member.default_value},"
        member_variable_set += f"\n        if isinstance({member.snake_case},{member.underlying_type}) {is_none_check}:"
        member_variable_set += f"\n            self._{member.snake_case} = _zivid.{settings_type}{path}.{member.camel_case}({member.snake_case})"
        member_variable_set += f"\n        else:"
        member_variable_set += f"\n            raise TypeError('Unsupported type, expected: {member.underlying_type}{can_be_none_error_message_part}, got {{value_type}}'.format(value_type=type({member.snake_case})))"

    for child_class in child_class_member_variables:
        signature_vars += f"{child_class.snake_case}={child_class.default_value},"
        member_variable_set += f"\n        if {child_class.snake_case} is None:"
        member_variable_set += f"\n            {child_class.snake_case} = zivid.{settings_type}{path}.{child_class.camel_case}()"
        member_variable_set += f"\n        if not isinstance({child_class.snake_case}, zivid.{settings_type}{path}.{child_class.camel_case}):"
        member_variable_set += f"\n            raise TypeError('Unsupported type: {{value}}'.format(value=type({child_class.snake_case})))"
        member_variable_set += (
            f"\n        self._{child_class.snake_case} = {child_class.snake_case}"
        )

    return """
    def __init__(
        self,
        {signature_vars}
    ):
        {member_variable_set}""".format(
        signature_vars=signature_vars, member_variable_set=member_variable_set
    )


def _create_eq_special_member_function(node_data, settings_type: str):
    if node_data.is_enum:
        return "\n    def __eq__(self,other):\n        if self.value == other.value:\n            return True\n        return False"
    member_variables_equality = list()
    member_variables = _get_member_variables(node_data, settings_type)
    child_class_member_variables = _get_child_class_member_variables(node_data)
    for member in member_variables:
        member_variables_equality.append(
            f"self._{member.snake_case} == other._{member.snake_case}"
        )
    for child in child_class_member_variables:
        member_variables_equality.append(
            f"self._{child.snake_case} == other._{child.snake_case}"
        )
    equality_logic = " and ".join(member_variables_equality)
    return """def __eq__(self, other):
        if (
            {equality_logic}
        ):
            return True
        return False""".format(
        equality_logic=equality_logic
    )


def _create_str_special_member_function(node_data, settings_type: str):
    str_content = ""
    member_variables = _get_member_variables(node_data, settings_type)
    standard_path = inflection.underscore(node_data.path.replace(".", "_"))
    if not standard_path:
        output = inflection.underscore(node_data.name)
    else:
        output = standard_path
    to_internal_function = f"to_internal_"
    to_internal = (
        f"{inflection.underscore(settings_type)}_{inflection.underscore(node_data.path).replace('.', '_')}"
        if node_data.path
        else f"{inflection.underscore(settings_type)}"
    )
    str_content = f"str(zivid._{inflection.underscore(settings_type).replace('.', '_')}_converter.{to_internal_function}{to_internal}(self))"

    return """def __str__(self):
            return {str_content}""".format(
        str_content=str_content
    )


def _leaf_underlying_type():
    return "some_type"


def _create_properties(node_data, settings_type: str):
    get_properties = "\n"
    set_properties = "\n"

    get_member_property_template = (
        "    @property\n    def {member}(self):\n        return self._{member}.value\n"
    )
    set_member_property_template = "    @{member}.setter\n    def {member}(self,value):\n        if isinstance(value,{expected_types}) {none_check}:\n            self._{member} = _zivid.{settings_type}{path}.{non_snake_member}(value)\n        else:\n            raise TypeError('Unsupported type, expected: {expected_types_str}, got {{value_type}}'.format(value_type=type(value)))\n"
    for member in node_data.member_variables:
        path = ".{path}".format(path=node_data.path,) if node_data.path else ""
        get_properties += get_member_property_template.format(member=member.snake_case)
        if member.is_optional:
            is_none_check = "or value is None"
            can_be_none_error_message_part = " or None"
        else:
            is_none_check = ""
            can_be_none_error_message_part = ""
        expected_types_str = " or ".join(member.underlying_type[1:-2].split(","))
        expected_types_str += can_be_none_error_message_part
        set_properties += set_member_property_template.format(
            member=member.snake_case,
            path=path,
            non_snake_member=member.camel_case,
            settings_type=settings_type,
            expected_types=member.underlying_type,
            none_check=is_none_check,
            can_be_none_error_message_part=can_be_none_error_message_part,
            expected_types_str=expected_types_str,
        )
    set_child_property_template = "    @{member}.setter\n    def {member}(self,value):\n        if not isinstance(value, zivid.{settings_type}{path}.{non_snake_member}):\n            raise TypeError('Unsupported type {{value}}'.format(value=type(value)))\n        self._{member} = value\n"
    get_child_property_template = (
        "    @property\n    def {member}(self):\n        return self._{member}\n"
    )
    for child in node_data.children:
        path = ".{path}".format(path=node_data.path,) if node_data.path else ""
        get_properties += get_child_property_template.format(
            member=inflection.underscore(child.name)
        )
        set_properties += set_child_property_template.format(
            member=inflection.underscore(child.name),
            path=path,
            non_snake_member=child.name,
            settings_type=settings_type,
        )
    return f"{get_properties}\n{set_properties}"


def _create_class_variables(node_data):
    sub_strings = []
    for enum_key, enum_value in node_data.enum_vars:
        sub_strings.append(
            f"{enum_key} = _zivid.capture_assistant.SuggestSettingsParameters.{node_data.path}.enum.{enum_value}"
        )
    return "\n    ".join(sub_strings)


def _create_class(node_data, settings_type: str):
    nested_classes = [
        _create_class(element, settings_type=settings_type)
        for element in node_data.children
    ]
    nested_classes_string = "\n".join(nested_classes)
    base_class = """
class {class_name}:
    {nested_classes}
    {class_variables}
    {init_special_member_function}
    {get_set_properties}
    {eq_special_member_function}
    {str_special_member_function}
""".format(
        class_name=node_data.name,
        nested_classes=nested_classes_string,
        class_variables=_create_class_variables(node_data),
        init_special_member_function=_create_init_special_member_function(
            node_data, settings_type=settings_type
        ),
        eq_special_member_function=_create_eq_special_member_function(
            node_data, settings_type=settings_type
        ),
        str_special_member_function=_create_str_special_member_function(
            node_data, settings_type=settings_type
        ),
        get_set_properties=_create_properties(node_data, settings_type=settings_type),
    )
    indented_lines = list()
    for line in base_class.splitlines():
        indented_lines.append("    " + line)
    return "\n".join(indented_lines)


def _recursion(current_class, indentation_level, parent_class=None):
    child_classes = list()
    print(f"this is class: {current_class.name}")
    print(dir(current_class))
    if not (hasattr(current_class, "valid_values") and hasattr(current_class, "enum")):
        for my_cls in _inner_classes_list(current_class):
            print("-----****")
            print(my_cls.name)
            if my_cls.name == "Experimental" and parent_class is None:
                continue
            child_classes.append(
                _recursion(
                    my_cls,
                    indentation_level=indentation_level + 1,
                    parent_class=current_class,
                )
            )
        is_leaf = not bool(_inner_classes_list(current_class))
    elif not hasattr(current_class, "_zivid_class"):
        is_leaf = True
    elif current_class._zivid_class.node_type in (
        "NodeType.leaf_value",
        "NodeType.leaf_data_model_list",
    ):
        is_leaf = True
    else:
        is_leaf = False

    member_variables = list()
    to_be_removed = list()
    for child in child_classes:
        if child.is_leaf:
            path = current_class.path.replace("/", ".")
            is_enum = True
            print("-------------------------------------")
            print(child.name)
            print(child._zivid_class.node_type)
            print(dir(child))
            print(dir(child._zivid_class))
            leaf_underlying_type = child._zivid_class.value_type
            print("found type!!!")
            print(leaf_underlying_type)

            member_variables.append(
                MemberVariable(
                    camel_case=child.name,
                    snake_case=inflection.underscore(child.name),
                    default_value="None",
                    underlying_type=leaf_underlying_type,
                    is_optional=child._zivid_class.is_optional,
                )
            )
            to_be_removed.append(child)
    child_classes = [
        element for element in child_classes if element not in to_be_removed
    ]

    if hasattr(current_class, "valid_values") and hasattr(current_class, "enum"):
        is_enum_class = True
        path = current_class.path.replace("/", ".")
        enum_default_value = current_class().value.name
        enum_vars = []
        members = [a for a in dir(current_class.enum)]
        for member in members:
            if str(member).startswith("__"):
                continue
            if str(member) == "name":
                continue
            enum_vars.append((member, getattr(current_class.enum, member).name))

    else:
        path = current_class.path.replace("/", ".")
        is_enum_class = False
        enum_vars = []
        enum_default_value = None

    print("\n------------------------------------------------")

    my_class = NodeData(
        name=current_class.name,
        is_leaf=is_leaf,
        is_enum=is_enum_class,
        enum_vars=enum_vars,
        enum_default_value=enum_default_value,
        path=path,
        children=child_classes,
        member_variables=member_variables,
        indentation_level=indentation_level,
        _zivid_class=current_class,
    )
    return my_class


def create_to_not_internal_converter(node_data, settings_type: str):
    temp_internal_name = "internal_{name}".format(
        name=inflection.underscore(node_data.name)
    )
    nested_converters = [
        create_to_not_internal_converter(element, settings_type=settings_type)
        for element in node_data.children
    ]
    nested_converters_string = "\n".join(nested_converters)
    return_class = "zivid.{settings_type}{path}".format(
        temp_internal_name=temp_internal_name,
        settings_type=settings_type,
        path=".{path}".format(path=node_data.path,) if node_data.path else "",
    )
    member_convert_logic = ""
    for member in node_data.member_variables:
        member_convert_logic += "{member} = {temp_internal_name}.{member}.value,".format(
            member=member.snake_case, temp_internal_name=temp_internal_name,
        )

    child_convert_logic = ""
    for child in node_data.children:
        child_convert_logic += "{child_name}=to_{child}({temp_internal_name}.{child_name}),".format(
            child_name=inflection.underscore(child.name),
            child=f"{inflection.underscore(settings_type)}_{inflection.underscore(node_data.path).replace('.', '_')}_{inflection.underscore(child.name)}"
            if node_data.path
            else f"{inflection.underscore(settings_type)}_{inflection.underscore(child.name)}",
            temp_internal_name=temp_internal_name,
        )

    base_function = """
def to_{path}(internal_{target_name}):
    return {return_class}({child_convert_logic} {member_convert_logic})
    
""".format(
        target_name=inflection.underscore(node_data.name),
        path=f"{inflection.underscore(settings_type).replace('.', '_')}_{inflection.underscore(node_data.path).replace('.', '_')}"
        if node_data.path
        else f"{inflection.underscore(settings_type).replace('.', '_')}",
        return_class=return_class,
        member_convert_logic=member_convert_logic,
        child_convert_logic=child_convert_logic,
    )
    indented_lines = list()
    for line in base_function.split("\n"):
        print(line)
        indented_lines.append("    " + line)
    return nested_converters_string + "\n" + "\n".join(indented_lines)


def create_to_internal_converter(node_data, settings_type: str):
    temp_internal_name = "internal_{name}".format(
        name=inflection.underscore(node_data.name)
    )
    nested_converters = [
        create_to_internal_converter(element, settings_type=settings_type)
        for element in node_data.children
    ]
    nested_converters_string = "\n".join(nested_converters)
    convert_member_logic = ""
    if node_data.member_variables:
        for member in node_data.member_variables:
            convert_member_logic += "\n    {temp_internal_name}.{member} = _zivid.{settings_type}{path}".format(
                temp_internal_name=temp_internal_name,
                member=member.snake_case,
                path=".{path}.{member_as_class}({name}.{member})".format(
                    path=node_data.path,
                    member_as_class=member.camel_case,
                    name=inflection.underscore(node_data.name),
                    member=member.snake_case,
                )
                if node_data.path
                else ".{member_as_class}({name}.{member})".format(
                    member_as_class=member.camel_case,
                    name=inflection.underscore(node_data.name),
                    member=member.snake_case,
                ),
                settings_type=settings_type,
            )

    convert_children_logic = ""
    if node_data.children:
        for child in node_data.children:
            convert_children_logic += "\n    {temp_internal_name}.{child_name} = to_internal_{child}({name}.{child_name})".format(
                temp_internal_name=temp_internal_name,
                child_name=inflection.underscore(child.name),
                child=f"{inflection.underscore(settings_type).replace('.', '_')}_{inflection.underscore(node_data.path).replace('.', '_')}_{inflection.underscore(child.name)}"
                if node_data.path
                else f"{inflection.underscore(settings_type).replace('.', '_')}_{inflection.underscore(child.name)}",
                name=inflection.underscore(node_data.name),
            )
    base_class = """
def to_internal_{path2}({target_name}):
    {temp_internal_name} = _zivid.{settings_type}{path}
    {convert_member_logic}
    {convert_children_logic}
    return {temp_internal_name}
""".format(
        target_name=inflection.underscore(node_data.name),
        convert_member_logic=convert_member_logic,
        convert_children_logic=convert_children_logic,
        path="." + node_data.path + "()" if node_data.path else "()",
        path2=f"{inflection.underscore(settings_type).replace('.', '_')}_{inflection.underscore(node_data.path).replace('.', '_')}"
        if node_data.path
        else f"{inflection.underscore(settings_type).replace('.', '_')}",
        temp_internal_name=temp_internal_name,
        settings_type=settings_type,
    )
    indented_lines = list()
    for line in base_class.splitlines():
        indented_lines.append("    " + line)
    return nested_converters_string + "\n" + "\n".join(indented_lines)


def common_class_generation(*, internal_class, settings_type, converter_import):
    data_model = _recursion(internal_class, indentation_level=0)
    with tempfile.NamedTemporaryFile(suffix=".py") as temp_file:
        temp_file = Path(temp_file.name)
        raw_text = _imports(
            internal=True,
            settings=True,
            additional_imports=(f"zivid.{converter_import}",),
        )
        raw_text += _create_class(data_model, settings_type=settings_type)

        new_lines = []
        for line in raw_text.splitlines():
            new_lines.append(line[4:])

        temp_file.write_text("\n".join(new_lines))
        print(temp_file.read_text())
        subprocess.check_output((f"black {temp_file}"), shell=True)
        print(temp_file.read_text())
        path_to_settings = (
            Path(__file__).resolve() / ".." / ".." / "zivid" / "settings__3d.py"
        ).resolve()
        path_to_settings.write_text(temp_file.read_text())


def common_to_internal_generation(*, internal_class, settings_type):
    data_model = _recursion(internal_class, indentation_level=0)
    with tempfile.NamedTemporaryFile(suffix=".py") as temp_file:
        temp_file = Path(temp_file.name)
        raw_text = _imports(internal=True, settings=False)
        raw_text += create_to_internal_converter(
            data_model, settings_type=settings_type
        )

        new_lines = []
        for line in raw_text.splitlines():
            new_lines.append(line[4:])

        temp_file.write_text("\n".join(new_lines))
        print(temp_file.read_text())
        subprocess.check_output((f"black {temp_file}"), shell=True)
        print(temp_file.read_text())
        path_to_settings = (
            Path(__file__).resolve() / ".." / ".." / "zivid" / "settings__3d.py"
        ).resolve()
        path_to_settings.write_text(temp_file.read_text())


def common_to_normal_generation(*, internal_class, settings_type):
    data_model = _recursion(internal_class, indentation_level=0)
    with tempfile.NamedTemporaryFile(suffix=".py") as temp_file:
        temp_file = Path(temp_file.name)
        raw_text = _imports(internal=False, settings=True)
        raw_text += create_to_not_internal_converter(
            data_model, settings_type=settings_type
        )

        new_lines = []
        for line in raw_text.splitlines():
            new_lines.append(line[4:])

        temp_file.write_text("\n".join(new_lines))
        print(temp_file.read_text())
        subprocess.check_output((f"black {temp_file}"), shell=True)
        print(temp_file.read_text())
        path_to_settings = (
            Path(__file__).resolve() / ".." / ".." / "zivid" / "settings__3d.py"
        ).resolve()
        path_to_settings.write_text(temp_file.read_text())



"""This file imports used classes, modules and packages."""
"""This file imports used classes, modules and packages."""
import inspect
from collections import namedtuple
from dataclasses import dataclass
from typing import Tuple
import subprocess
from common import (
    _create_class,
    _imports,
    _recursion,
    common_class_generation,
)


def start_traverse():
    from _zivid._zivid import Settings

    common_class_generation(
        internal_class=Settings,
        settings_type="Settings",
        converter_import="_settings_converter",
    )


# def start_traverse():

#     import tempfile
#     from pathlib import Path
#
#     data_model = _recursion(Settings, indentation_level=0)
#     with tempfile.NamedTemporaryFile(suffix=".py") as temp_file:
#         temp_file = Path(temp_file.name)
#         raw_text = _imports(
#             internal=True,
#             settings=True,
#             additional_imports=("zivid._settings_converter",),
#         )
#         raw_text += _create_settings_py(data_model)
#
#         new_lines = []
#         for line in raw_text.splitlines():
#             new_lines.append(line[4:])
#
#         temp_file.write_text("\n".join(new_lines))
#         print(temp_file.read_text())
#         subprocess.check_output((f"black {temp_file}"), shell=True)
#         print(temp_file.read_text())
#         path_to_settings = (
#             Path(__file__).resolve() / ".." / ".." / "zivid" / "settings__3d.py"
#         ).resolve()
#         path_to_settings.write_text(temp_file.read_text())
#
#
# def _create_settings_py(data_model):
#     return _create_class(data_model, settings_type="Settings")
#



"""This file imports used classes, modules and packages."""
import inspect
from collections import namedtuple
from dataclasses import dataclass
from typing import Tuple
import subprocess


from _zivid.common import (
    _create_class,
    _imports,
    _recursion,
    common_class_generation,
)


def start_traverse():
    from _zivid._zivid import Settings2D

    common_class_generation(
        internal_class=Settings2D,
        settings_type="Settings2D",
        converter_import="_settings_2d_converter",
    )



import generate_settings
import generate_to_internal_settings_converter
import generate_to_settings_converter

#
# import _zivid.generate_camera_info
# import _zivid.generate_to_internal_camera_info_converter
# import _zivid.generate_to_camera_info_converter
#
import generate_settings_2d
import generate_to_internal_settings_2d_converter
import generate_to_settings_2d_converter

#
# import _zivid.generate_frame_info
# import _zivid.generate_to_frame_info_converter
# import _zivid.generate_to_frame_internal_info_converter
#
# import _zivid.generate_camera_state
# import _zivid.generate_to_camera_state_converter
# import _zivid.generate_to_internal_camera_state_converter
#
# import _zivid.generate_suggest_settings_parameters
# import _zivid.generate_to_internal_suggest_settings_parameters_converter
# import _zivid.generate_to_suggest_settings_parameters_converter


def generate():
    pass
    # generate_settings.start_traverse()
    # generate_to_settings_converter.start_traverse()
    # generate_to_internal_settings_converter.start_traverse()

    # _zivid.generate_camera_info.start_traverse()
    # _zivid.generate_to_camera_info_converter.start_traverse()
    # _zivid.generate_to_internal_camera_info_converter.start_traverse()

    # _zivid.generate_frame_info.start_traverse()
    # _zivid.generate_to_frame_info_converter.start_traverse()
    # _zivid.generate_to_frame_internal_info_converter.start_traverse()

    # _zivid.generate_camera_state.start_traverse()
    # _zivid.generate_to_camera_state_converter.start_traverse()
    # _zivid.generate_to_internal_camera_state_converter.start_traverse()

    # generate_settings_2d.start_traverse()
    # generate_to_settings_2d_converter.start_traverse()
    generate_to_internal_settings_2d_converter.start_traverse()

    # needs redoing
    # _zivid.generate_suggest_settings_parameters.start_traverse()
    # _zivid.generate_to_suggest_settings_parameters_converter.start_traverse()
    # _zivid.generate_to_internal_suggest_settings_parameters_converter.start_traverse()


generate()



"""This file imports used classes, modules and packages."""
import inspect
from collections import namedtuple
from dataclasses import dataclass
from typing import Tuple
import subprocess
from _zivid.common import (
    _create_class,
    _imports,
    _recursion,
    common_to_internal_generation,
)
import tempfile
from pathlib import Path
import inflection
from _zivid._zivid import Settings


def start_traverse():
    common_to_internal_generation(internal_class=Settings, settings_type="Settings")



"""Auto generated, do not edit."""
# pylint: disable=too-many-arguments,missing-class-docstring,missing-function-docstring
import _zivid
import zivid
import zivid._camera_info_converter


class CameraInfo:
    class Revision:
        def __init__(
            self,
            major=_zivid.CameraInfo().Revision().Major().value,
            minor=_zivid.CameraInfo().Revision().Minor().value,
        ):

            if isinstance(major, (int,)):
                self._major = _zivid.CameraInfo.Revision.Major(major)
            else:
                raise TypeError(
                    "Unsupported type, expected: (int,), got {value_type}".format(
                        value_type=type(major)
                    )
                )
            if isinstance(minor, (int,)):
                self._minor = _zivid.CameraInfo.Revision.Minor(minor)
            else:
                raise TypeError(
                    "Unsupported type, expected: (int,), got {value_type}".format(
                        value_type=type(minor)
                    )
                )

        @property
        def major(self):
            return self._major.value

        @property
        def minor(self):
            return self._minor.value

        @major.setter
        def major(self, value):
            if isinstance(value, (int,)):
                self._major = _zivid.CameraInfo.Revision.Major(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: int, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        @minor.setter
        def minor(self, value):
            if isinstance(value, (int,)):
                self._minor = _zivid.CameraInfo.Revision.Minor(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: int, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        def __eq__(self, other):
            if self._major == other._major and self._minor == other._minor:
                return True
            return False

        def __str__(self):
            return str(
                zivid._camera_info_converter.to_internal_camera_info_revision(self)
            )

    class UserData:
        def __init__(
            self, max_size_bytes=_zivid.CameraInfo().UserData().MaxSizeBytes().value,
        ):

            if isinstance(max_size_bytes, (int,)):
                self._max_size_bytes = _zivid.CameraInfo.UserData.MaxSizeBytes(
                    max_size_bytes
                )
            else:
                raise TypeError(
                    "Unsupported type, expected: (int,), got {value_type}".format(
                        value_type=type(max_size_bytes)
                    )
                )

        @property
        def max_size_bytes(self):
            return self._max_size_bytes.value

        @max_size_bytes.setter
        def max_size_bytes(self, value):
            if isinstance(value, (int,)):
                self._max_size_bytes = _zivid.CameraInfo.UserData.MaxSizeBytes(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: int, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        def __eq__(self, other):
            if self._max_size_bytes == other._max_size_bytes:
                return True
            return False

        def __str__(self):
            return str(
                zivid._camera_info_converter.to_internal_camera_info_user_data(self)
            )

    def __init__(
        self,
        firmware_version=_zivid.CameraInfo().FirmwareVersion().value,
        model_name=_zivid.CameraInfo().ModelName().value,
        serial_number=_zivid.CameraInfo().SerialNumber().value,
        revision=None,
        user_data=None,
    ):

        if isinstance(firmware_version, (str,)):
            self._firmware_version = _zivid.CameraInfo.FirmwareVersion(firmware_version)
        else:
            raise TypeError(
                "Unsupported type, expected: (str,), got {value_type}".format(
                    value_type=type(firmware_version)
                )
            )
        if isinstance(model_name, (str,)):
            self._model_name = _zivid.CameraInfo.ModelName(model_name)
        else:
            raise TypeError(
                "Unsupported type, expected: (str,), got {value_type}".format(
                    value_type=type(model_name)
                )
            )
        if isinstance(serial_number, (str,)):
            self._serial_number = _zivid.CameraInfo.SerialNumber(serial_number)
        else:
            raise TypeError(
                "Unsupported type, expected: (str,), got {value_type}".format(
                    value_type=type(serial_number)
                )
            )
        if revision is None:
            revision = zivid.CameraInfo.Revision()
        if not isinstance(revision, zivid.CameraInfo.Revision):
            raise TypeError("Unsupported type: {value}".format(value=type(revision)))
        self._revision = revision
        if user_data is None:
            user_data = zivid.CameraInfo.UserData()
        if not isinstance(user_data, zivid.CameraInfo.UserData):
            raise TypeError("Unsupported type: {value}".format(value=type(user_data)))
        self._user_data = user_data

    @property
    def firmware_version(self):
        return self._firmware_version.value

    @property
    def model_name(self):
        return self._model_name.value

    @property
    def serial_number(self):
        return self._serial_number.value

    @property
    def revision(self):
        return self._revision

    @property
    def user_data(self):
        return self._user_data

    @firmware_version.setter
    def firmware_version(self, value):
        if isinstance(value, (str,)):
            self._firmware_version = _zivid.CameraInfo.FirmwareVersion(value)
        else:
            raise TypeError(
                "Unsupported type, expected: str, got {value_type}".format(
                    value_type=type(value)
                )
            )

    @model_name.setter
    def model_name(self, value):
        if isinstance(value, (str,)):
            self._model_name = _zivid.CameraInfo.ModelName(value)
        else:
            raise TypeError(
                "Unsupported type, expected: str, got {value_type}".format(
                    value_type=type(value)
                )
            )

    @serial_number.setter
    def serial_number(self, value):
        if isinstance(value, (str,)):
            self._serial_number = _zivid.CameraInfo.SerialNumber(value)
        else:
            raise TypeError(
                "Unsupported type, expected: str, got {value_type}".format(
                    value_type=type(value)
                )
            )

    @revision.setter
    def revision(self, value):
        if not isinstance(value, zivid.CameraInfo.Revision):
            raise TypeError("Unsupported type {value}".format(value=type(value)))
        self._revision = value

    @user_data.setter
    def user_data(self, value):
        if not isinstance(value, zivid.CameraInfo.UserData):
            raise TypeError("Unsupported type {value}".format(value=type(value)))
        self._user_data = value

    def __eq__(self, other):
        if (
            self._firmware_version == other._firmware_version
            and self._model_name == other._model_name
            and self._serial_number == other._serial_number
            and self._revision == other._revision
            and self._user_data == other._user_data
        ):
            return True
        return False

    def __str__(self):
        return str(zivid._camera_info_converter.to_internal_camera_info(self))



"""Auto generated, do not edit."""
import zivid
import _zivid


def to_frame_info_software_version(internal_software_version):
    return zivid.FrameInfo.SoftwareVersion(core=internal_software_version.core.value,)


def to_frame_info(internal_frame_info):
    return zivid.FrameInfo(
        software_version=to_frame_info_software_version(
            internal_frame_info.software_version
        ),
        time_stamp=internal_frame_info.time_stamp.value,
    )


def to_internal_frame_info_software_version(software_version):
    internal_software_version = _zivid.FrameInfo.SoftwareVersion()

    internal_software_version.core = _zivid.FrameInfo.SoftwareVersion.Core(
        software_version.core
    )

    return internal_software_version


def to_internal_frame_info(frame_info):
    internal_frame_info = _zivid.FrameInfo()

    internal_frame_info.time_stamp = _zivid.FrameInfo.TimeStamp(frame_info.time_stamp)

    internal_frame_info.software_version = to_internal_frame_info_software_version(
        frame_info.software_version
    )
    return internal_frame_info
